#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
# Zyxel WLC metrics plugin
#
# Author: Th.L.
# Date  : 2018-01-16
#
#

# key a       green      11/a       21/a       31/a       41/a       12/a       22/a       32/a       42/a
colors_a = ['#80F000', '#a500ff', '#ffc600', '#00ffb2', '#0075ff', '#cc00ff', '#ffd600', '#00ffff', '#0047ff',
            # 13/a       23/a       33/a       43/a       14/a       24/a       34/a       44/a       15/a
            '#f900ff', '#ffed00', '#00e8ff', '#000aff', '#ff4c00', '#e2ff00', '#00d1ff', '#4200ff', '#ff7a00',
            # 25/a       35/a       45/a       16/a       26/a       36/a       46/a       51/a       52/a
            '#bcff00', '#00b2ff', '#6000ff', '#ffa000', '#7fff00', '#0093ff', '#7f00ff', '#7f7f7f', '#7f4a26',
            # 53/a
            '#8c531c']
# key b       green      21/b       31/b       41/b       12/b       22/b       32/b       42/b       13/b
colors_b = ['#80F000', '#c966ff', '#cc9f00', '#00cc8e', '#66acff', '#e066ff', '#ccab00', '#00cccc', '#6690ff',
            # 23/b       33/b       43/b       14/b       24/b       34/b       44/b       15/b       25/b
            '#fb66ff', '#ccbd00', '#00b9cc', '#666cff', '#ff9366', '#b5cc00', '#00a7cc', '#8d66ff', '#ffaf66',
            # 35/b       45/b       16/b       26/b       36/b       46/b       51/b       52/b       53/b
            '#96cc00', '#008ecc', '#a066ff', '#ffc666', '#66cc00', '#0076cc', '#b266ff', '#7f7f7f', '#7f5f49',
            '#8c6a48']

#####################################################################################################################
#
# define units for zyxel wlc perfdata
#
#####################################################################################################################

unit_info['wlcaps'] = {
    'title': _('APs'),
    'symbol': '',
    'render': lambda v: '%d' % v,
    'stepping': 'integer',  # for vertical graph labels
}

unit_info['clients'] = {
    'title': _('Clients'),
    'symbol': '',
    'render': lambda v: '%d' % v,
    'stepping': 'integer',  # for vertical graph labels
}

unit_info['frames/s'] = {
    'title': _('frames/s'),
    'description': _('Frequency (displayed in events/s)'),
    'symbol': _('/s'),
    'render': lambda v: '%s%s' % (drop_dotzero(v), _('/s')),
}

#####################################################################################################################
#
# define metrics for zyxel wlc perfdata
#
#####################################################################################################################


metric_info['zyxel_capwap_onlineaps'] = {
    'title': _('online APs'),
    'unit': 'wlcaps',
    'color': colors_a[0],
}

metric_info['zyxel_capwap_offlineaps'] = {
    'title': _('offline APs'),
    'unit': 'wlcaps',
    'color': colors_a[1],
}

metric_info['zyxel_capwap_unmanagedaps'] = {
    'title': _('unmanaged APs'),
    'unit': 'wlcaps',
    'color': colors_a[2],
}

metric_info['zyxel_capwap_wlanclients'] = {
    'title': _('total WLAN clients'),
    'unit': 'clients',
    'color': colors_a[3],
}

metric_info['zyxel_wlancontrolleraptable_tx_frames'] = {
    'title': _('tx frames'),
    'unit': 'frames/s',
    'color': colors_a[0],
}

metric_info['zyxel_wlancontrolleraptable_rx_frames'] = {
    'title': _('rx frames'),
    'unit': 'frames/s',
    'color': colors_a[1],
}

metric_info['zyxel_wlancontrolleraptable_number_of_clients'] = {
    'title': _('# of clients'),
    'unit': 'clients',
    'color': colors_a[3],
}

metric_info['zyxel_cpu_cpu_usage'] = {
    'title': _('CPU Usage'),
    'unit': '%',
    'color': colors_a[0],
}

metric_info['zyxel_mem_mem_usage'] = {
    'title': _('Memeory Usage'),
    'unit': '%',
    'color': colors_a[0],
}

metric_info['zyxel_flash_flash_usage'] = {
    'title': _('Flash Usage'),
    'unit': '%',
    'color': colors_a[0],
}

metric_info['zyxel_foreign_aps_unclassified_aps'] = {
    'title': _('unclassified APs'),
    'unit': 'wlcaps',
    'color': colors_a[0],
}

metric_info['zyxel_foreign_aps_rogue_aps'] = {
    'title': _('rogue APs'),
    'unit': 'wlcaps',
    'color': colors_a[1],
}

metric_info['zyxel_foreign_aps_friendly_aps'] = {
    'title': _('friendly APs'),
    'unit': 'wlcaps',
    'color': colors_a[2],
}

######################################################################################################################
#
# map zyxel wlc perfdata to metric, not really necessary but makes sure to use the right metrics
#
######################################################################################################################

check_metrics['check_mk-zyxel_capwap'] = {
    'onlineaps': {'name': 'zyxel_capwap_onlineaps', },
    'offlineaps': {'name': 'zyxel_capwap_offlineaps', },
    'unmanagedaps': {'name': 'zyxel_capwap_unmanagedaps', },
    'wlanclients': {'name': 'zyxel_capwap_wlanclients', },
}

check_metrics['check_mk-zyxel_wlancontrolleraptable'] = {
    'tx_frames': {'name': 'zyxel_wlancontrolleraptable_tx_frames', },
    'rx_frames': {'name': 'zyxel_wlancontrolleraptable_rx_frames', },
    'number_of_clients': {'name': 'zyxel_wlancontrolleraptable_number_of_clients', },
}

check_metrics['check_mk-zyxel_cpu'] = {
    'cpu_usage': {'name': 'zyxel_cpu_cpu_usage', },
}

check_metrics['check_mk-zyxel_mem'] = {
    'mem_usage': {'name': 'zyxel_mem_mem_usage', },
}

check_metrics['check_mk-zyxel_flash'] = {
    'flash_usage': {'name': 'zyxel_flash_flash_usage', },
}

check_metrics['check_mk-zyxel_foreign_aps'] = {
    'unclassified_aps': {'name': 'zyxel_foreign_aps_unclassified_aps', },
    'rogue_aps': {'name': 'zyxel_foreign_aps_rogue_aps', },
    'friendly_aps': {'name': 'zyxel_foreign_aps_friendly_aps', },
}

######################################################################################################################
#
# how to graph perdata for zyxel wlc
#
######################################################################################################################

# zyxel capwap
graph_info.append({
    'title': _('AP count'),
    'metrics': [
        ('zyxel_capwap_onlineaps', 'line'),
        ('zyxel_capwap_offlineaps', 'line'),
        ('zyxel_capwap_unmanagedaps', 'line'),
    ],
    'range': (0, 'zyxel_capwap_onlineaps:max')
})

graph_info.append({
    'title': _('WLAN Clients on WLC'),
    'metrics': [
        ('zyxel_capwap_wlanclients', 'line'),
    ],
    'range': (0, 'zyxel_capwap_wlanclients:max'),
})

# zyxel wlancontrolleraptable
graph_info.append({
    'title': _('tx/rx frames/s'),
    'metrics': [
        ('zyxel_wlancontrolleraptable_tx_frames', 'line'),
        ('zyxel_wlancontrolleraptable_rx_frames', '-line'),
    ],
    'range': (0, 'zyxel_wlancontrolleraptable_tx_frames:max')
})

graph_info.append({
    'title': _('WLAN Clients on AP'),
    'metrics': [
        ('zyxel_wlancontrolleraptable_number_of_clients', 'line'),
    ],
    'range': (0, 'zyxel_wlancontrolleraptable_number_of_clients:max'),
})

graph_info.append({
    'title': _('CPU usage'),
    'metrics': [
        ('zyxel_cpu_cpu_usage', 'line'),
    ],
    'range': (0, 110),
    'scalars': [
        'zyxel_cpu_cpu_usage:crit',
        'zyxel_cpu_cpu_usage:warn',
    ],
})

graph_info.append({
    'title': _('Memory usage'),
    'metrics': [
        ('zyxel_mem_mem_usage', 'line'),
    ],
    'range': (0, 110),
    'scalars': [
        'zyxel_mem_mem_usage:crit',
        'zyxel_mem_mem_usage:warn',
    ],
})

graph_info.append({
    'title': _('Memory usage'),
    'metrics': [
        ('zyxel_flash_flash_usage', 'line'),
    ],
    'range': (0, 110),
    'scalars': [
        'zyxel_flash_flash_usage:crit',
        'zyxel_flash_flash_usage:warn',
    ],
})

# zyxel foreign_aps
graph_info.append({
    'title': _('Foreign AP count'),
    'metrics': [
        ('zyxel_foreign_aps_unclassified_aps', 'line'),
        ('zyxel_foreign_aps_rogue_aps', 'line'),
        ('zyxel_foreign_aps_friendly_aps', 'line'),
    ],
    'range': (0, 'zyxel_foreign_aps_unclassified_aps:max')
})

######################################################################################################################
#
# define perf-o-meter for zyxel wlc
#
######################################################################################################################

# zyxel capwap
perfometer_info.append(('stacked', [
    {
        'type': 'linear',
        'segments': ['zyxel_capwap_onlineaps'],
        'total': 100,
    },
    {
        'type': 'linear',
        'segments': ['zyxel_capwap_wlanclients'],
        'total': 100,
    }
]))

# zyxel wlancontrolleraptable
perfometer_info.append(('dual', [
    {
        'type': 'logarithmic',
        'metric': 'zyxel_wlancontrolleraptable_tx_frames',
        'half_value': 5000000,
        'exponent': 5,
    },
    {
        'type': 'logarithmic',
        'metric': 'zyxel_wlancontrolleraptable_rx_frames',
        'half_value': 5000000,
        'exponent': 5,
    }
]))

# zyxel zyxel_cpu
perfometer_info.append(
    {
        'type': 'linear',
        'segments': ['zyxel_cpu_cpu_usage'],
        'total': 100,
    }
)

# zyxel zyxel_mem
perfometer_info.append(
    {
        'type': 'linear',
        'segments': ['zyxel_mem_mem_usage'],
        'total': 100,
    }
)

# zyxel zyxel_flash
perfometer_info.append(
    {
        'type': 'linear',
        'segments': ['zyxel_flash_flash_usage'],
        'total': 100,
    }
)

# zyxel foreign aps
perfometer_info.append(
    {
        'type': 'linear',
        'segments': ['zyxel_foreign_aps_unclassified_aps',
                     'zyxel_foreign_aps_rogue_aps',
                     'zyxel_foreign_aps_friendly_aps'],
        'total': 100,
    },
)
